@@ -16,26 +16,30 @@
 
 """Recursive diff and patch for nested structures."""
 
-from pickle import dumps
 from warnings import warn
 
 import nested_diff.handlers
 
 __all__ = ['Differ', 'Iterator', 'Patcher', 'diff', 'patch']
 
-__version__ = '0.12'
+__version__ = '0.14'
 __author__ = 'Michael Samoglyadov'
 __license__ = 'Apache License, Version 2.0'
 __website__ = 'https://github.com/mr-mixas/Nested-Diff.py'
 
-_DEFAULT_HANDLER = nested_diff.handlers.TypeHandler()
+DEFAULT_HANDLER = nested_diff.handlers.TypeHandler()
 
-_TYPE_HANDLERS = (
+TYPE_HANDLERS = (
     nested_diff.handlers.DictHandler(),
     nested_diff.handlers.ListHandler(),
     nested_diff.handlers.TupleHandler(),
     nested_diff.handlers.SetHandler(),
     nested_diff.handlers.FrozenSetHandler(),
+
+    nested_diff.handlers.IntHandler(),
+    nested_diff.handlers.FloatHandler(),
+    nested_diff.handlers.StrHandler(),
+    nested_diff.handlers.BytesHandler(),
 )
 
 
@@ -96,7 +100,7 @@ class Differ(object):
 
     """
 
-    default_handler = _DEFAULT_HANDLER
+    default_differ = DEFAULT_HANDLER.diff
 
     def __init__(self, A=True, N=True, O=True, R=True, U=True,  # noqa: E501 E741 N803
                  trimR=False, handlers=None):
@@ -120,35 +124,33 @@ class Differ(object):
         self.op_u = U
         self.op_trim_r = trimR
 
-        self.__differs = {}
+        self._differs = {}
 
-        for handler in _TYPE_HANDLERS if handlers is None else handlers:
+        for handler in TYPE_HANDLERS if handlers is None else handlers:
             self.set_handler(handler)
 
     def diff(self, a, b):
         """
-        Return diff for two arbitrary objects.
+        Return equality flag and diff for two arbitrary objects.
 
-        This method calls registered handler for each diffed values pair
-        according their type. Default handler called for objects with different
-        or not registered types.
+        This method calls registered handler according diffed objects type.
+        Default handler called for objects with different types or when no
+        handler registered for such type.
 
         :param a: First object to diff.
         :param b: Second object to diff.
 
         """
+        if a is b:
+            return True, {'U': a} if self.op_u else {}
+
         if a.__class__ is b.__class__:
-            # it's faster to compare pickled dumps and dig differences
-            # afterwards than recursively diff each pair of objects
-            if a is b or dumps(a, -1) == dumps(b, -1):
-                return {'U': a} if self.op_u else {}
-
             try:
-                return self.__differs[a.__class__](self, a, b)
+                return self._differs[a.__class__](self, a, b)
             except KeyError:
-                return self.default_handler.diff(self, a, b)
+                pass
 
-        return self.default_handler.diff(self, a, b)
+        return self.default_differ(self, a, b)
 
     def set_handler(self, handler):
         """
@@ -157,13 +159,13 @@ class Differ(object):
         :param handler: handlers.TypeHandler.
 
         """
-        self.__differs[handler.handled_type] = handler.diff
+        self._differs[handler.handled_type] = handler.diff
 
 
 class Patcher(object):
     """Patch objects using nested diff."""
 
-    default_handler = _DEFAULT_HANDLER
+    default_patcher = DEFAULT_HANDLER.patch
 
     def __init__(self, handlers=None):
         """
@@ -172,9 +174,9 @@ class Patcher(object):
         :param handlers: List of type handlers.
 
         """
-        self.__patchers = {}
+        self._patchers = {}
 
-        for handler in _TYPE_HANDLERS if handlers is None else handlers:
+        for handler in TYPE_HANDLERS if handlers is None else handlers:
             self.set_handler(handler)
 
         if handlers is None:
@@ -198,11 +200,12 @@ class Patcher(object):
                 type_ = ndiff['D'].__class__
 
             try:
-                return self.__patchers[type_](self, target, ndiff)
+                return self._patchers[type_](self, target, ndiff)
             except KeyError:
-                raise NotImplementedError('unsupported diff type') from None
+                raise ValueError('unsupported patch type: '
+                                 + type_.__name__) from None
 
-        return self.default_handler.patch(self, target, ndiff)
+        return self.default_patcher(self, target, ndiff)
 
     def set_handler(self, handler):
         """
@@ -211,13 +214,13 @@ class Patcher(object):
         :param handler: handlers.TypeHandler.
 
         """
-        self.__patchers[handler.handled_type] = handler.patch
+        self._patchers[handler.handled_type] = handler.patch
 
 
 class Iterator(object):
     """Nested diff iterator."""
 
-    default_handler = _DEFAULT_HANDLER
+    default_iterator = DEFAULT_HANDLER.iterate_diff
 
     def __init__(self, handlers=None, sort_keys=False):
         """
@@ -230,9 +233,9 @@ class Iterator(object):
 
         """
         self.sort_keys = sort_keys
-        self.__iterators = {}
+        self._iterators = {}
 
-        for handler in _TYPE_HANDLERS if handlers is None else handlers:
+        for handler in TYPE_HANDLERS if handlers is None else handlers:
             self.set_handler(handler)
 
     def _get_iterator(self, ndiff):
@@ -243,19 +246,18 @@ class Iterator(object):
 
         """
         if 'E' in ndiff:
-            return self.default_handler.iterate_diff(self, ndiff)
+            return self.default_iterator(self, ndiff)
 
         try:
-            return self.__iterators[ndiff['D'].__class__](self, ndiff)
+            return self._iterators[ndiff['D'].__class__](self, ndiff)
         except KeyError:
-            return self.default_handler.iterate_diff(self, ndiff)
+            return self.default_iterator(self, ndiff)
 
-    def iterate(self, ndiff, depth=0):
+    def iterate(self, ndiff):
         """
-        Yield tuples with diff, key, subdiff and depth for each nested diff.
+        Yield tuples with diff, key and subdiff for each nested diff.
 
         :param ndiff: nested diff.
-        :param depth: depth initial value (for use in subiterators).
 
         """
         stack = [self._get_iterator(ndiff)]
@@ -265,15 +267,13 @@ class Iterator(object):
                 ndiff, key, subdiff = next(stack[-1])
             except StopIteration:
                 stack.pop()
-                depth -= 1
                 continue
 
-            yield ndiff, key, subdiff, depth
+            yield ndiff, key, subdiff
 
             if subdiff is None:
                 continue
 
-            depth += 1
             stack.append(self._get_iterator(subdiff))
 
     def set_handler(self, handler):
@@ -283,7 +283,7 @@ class Iterator(object):
         :param handler: handlers.TypeHandler.
 
         """
-        self.__iterators[handler.handled_type] = handler.iterate_diff
+        self._iterators[handler.handled_type] = handler.iterate_diff
 
 
 def diff(a, b, text_diff_ctx=-1, **kwargs):
@@ -309,7 +309,7 @@ def diff(a, b, text_diff_ctx=-1, **kwargs):
         differ.set_handler(nested_diff.handlers.TextHandler(
             context=text_diff_ctx))
 
-    return differ.diff(a, b)
+    return differ.diff(a, b)[1]
 
 
 def patch(target, ndiff, **kwargs):
