@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2019-2021 Michael Samoglyadov
+# Copyright 2019-2022 Michael Samoglyadov
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -17,6 +17,8 @@
 """Patch tool for nested diff."""
 
 import argparse
+import sys
+
 import nested_diff.cli
 
 
@@ -24,16 +26,31 @@ class App(nested_diff.cli.App):
     """Patch tool for nested data structures."""
 
     def dump(self, file_, data, fmt):
+        """
+        Dump patched data using apropriate format.
+
+        :param file_: File object to dump.
+        :param data: Data to dump.
+        :param fmt: Format used for dump, one of `self.supported_ofmts`.
+
+        """
         if fmt == 'auto':
             fmt = self.guess_fmt(file_, 'json')
 
         super().dump(file_, data, fmt)
 
     def get_positional_args_parser(self):
+        """Return parser for positional part of CLI args."""
         parser = super().get_positional_args_parser()
 
         parser.add_argument('target_file')
-        parser.add_argument('patch_file', type=argparse.FileType())
+        parser.add_argument(
+            'patch_file',
+            nargs='?',
+            default=sys.stdin,
+            type=argparse.FileType(),
+            help='optional, STDIN used when omitted',
+        )
 
         return parser
 
@@ -49,6 +66,7 @@ class App(nested_diff.cli.App):
         return nested_diff.patch(target, diff)
 
     def run(self):
+        """Patch app object entry point."""
         patched = self.patch(
             self.load(argparse.FileType()(self.args.target_file)),
             self.load(self.args.patch_file),
